<h1>[% thing.title %]</h1>

<p>[% thing.description %]</p>

<h2>Albums</h2>
[% FOREACH a = thing.albums %]
	<p><a href="[% config.url %]?[% a.moniker %]=[% a.id %]">[% a.title %]</a><br>
	[% a.description %]</p>
[% END %]
</h2>

