<h1><a href="[% config.url %]/artist/[% thing.artist.id %]">[% thing.artist.title %]</a><br>
[% thing.title %]</h1>


<p>[% thing.description %]</p>

<h2>Tracks</h2>
[% counter = 1 %]
<p>[% FOREACH t = thing.tracks %]
	
	[% IF t == highlight %]<a name="track[% t.id %]"><b>[% END %][% counter %]. [% t.title %][% '</b></a>' IF t == highlight %]<br>
	[% counter = counter + 1 %]
	
[% END %]</p>
</h2>

<p>More <a href="[% config.url %]/genre/[% thing.genre.id %]">[% thing.genre.title %]</a>.</p>

