<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
	"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd">

<html>
<head>
<title>[% config.site_title %] * [% title %]</title>
<link rel="stylesheet" href="/stylesheets/demo.css">
</head>
<body>

<p class="small">
<a href="[% config.url %]"><b>home</b></a>
[% FOREACH c = factory.classes %] | <a href="[% config.url %]?type=[% c %]">[% factory.plural(c) %]</a>[% END %]
</p>
